package bakalarka.mendelu.cz.bakamsg;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.firebase.ui.database.FirebaseListAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.ServerValue;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import bakalarka.mendelu.cz.bakamsg.dataModels.Message;

public class ChatWindowWithFriend extends AppCompatActivity {

    DatabaseReference database = FirebaseDatabase.getInstance().getReference();
    final FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
    Button sendButton;
    EditText chatMessage;
    ListView listOfMessages;
    String friendUID;
    String friendNick;
    String mergedID;
    final Context context = this;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chat_window_with_friend);

        sendButton = (Button) findViewById(R.id.buttonDiscussion);
        chatMessage = (EditText) findViewById(R.id.editTextDiscussion);
        listOfMessages = (ListView) findViewById(R.id.listOfChatMessages);

        Intent i = getIntent();
        friendNick = i.getStringExtra("friendNick");
        friendUID = i.getStringExtra("friendUID");
        getSupportActionBar().setTitle("Chat with "+i.getStringExtra("chatWith"));

        String ui1 = user.getUid();
        String ui2 = friendUID;
        /**
         * Příprava na převod časového razítka ze serveru na lokální čas.
         */
        final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        final String timezoneID = TimeZone.getDefault().getID();
        dateFormat.setTimeZone(TimeZone.getTimeZone(timezoneID));

        int a = ui1.compareTo(ui2);
        if (a < 0){
            Log.w("s1 je před s2",ui1+ui2);
            mergedID = ui1+ui2;
        } else {
            Log.w("s2 je před s1",ui2+ui1);
            mergedID = ui2+ui1;
        }

        sendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!chatMessage.getText().toString().isEmpty()){
                    sendMessage(chatMessage.getText().toString(),friendNick,friendUID,mergedID);
                }
            }
        });
        /**
         * Naplnění dat z Firebase přes Firebase adapter do připraveného listu.
         */
        final FirebaseListAdapter<Message> adapter =
            new FirebaseListAdapter<Message>(this, Message.class, R.layout.list_chat_messages, database.child("privateMessages").child(mergedID).orderByChild("date")) {
                @Override
                protected void populateView(View v, Message model, int position) {
                    if ((model.getUser1ID().equals(user.getUid()) && model.getUser2ID().equals(friendUID))){
                        ((TextView)v.findViewById(R.id.textViewListMessage)).setText(model.getText());
                        ((TextView)v.findViewById(R.id.textViewListNick)).setText("From: "+model.getMyNick());
                        ((TextView)v.findViewById(R.id.textViewListDate)).setText("["+dateFormat.format(new Date (model.getDate()))+"]");
                    } else if ((model.getUser1ID().equals(friendUID) && model.getUser2ID().equals(user.getUid()))){
                        ((TextView)v.findViewById(R.id.textViewListMessage)).setText(model.getText());
                        ((TextView)v.findViewById(R.id.textViewListNick)).setText("From: "+model.getMyNick());
                        ((TextView)v.findViewById(R.id.textViewListDate)).setText("["+dateFormat.format(new Date (model.getDate()))+"]");
                    }
                }
            };

        listOfMessages.setAdapter(adapter);
        /**
         * Editace existujících zpráv, jichž je přihlášený uživatel vlastníkem.
         */
        listOfMessages.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, final int position, long id) {
                final Message msg = adapter.getItem(position);
                if (msg.getUser1ID().equals(user.getUid())){
                    AlertDialog.Builder editMessage = new AlertDialog.Builder(context);
                    final EditText edittext = new EditText(context);
                    editMessage.setMessage("Original: "+msg.getText());
                    editMessage.setView(edittext);
                    editMessage.setPositiveButton("Edit", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                            if(!edittext.getText().toString().equals("")){
                                DatabaseReference msgRef = database.child("privateMessages/"+mergedID+"/"+msg.getMsgKey()+"/text");
                                msgRef.setValue(edittext.getText().toString());
                            } else {
                                Toast.makeText(getApplicationContext(), "Message has not been edited.", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                    editMessage.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                        }
                    });
                    editMessage.show();
                }
            }
        });
    }
    /**
     * Odeslání zprávy do Firebase.
     */
    public void sendMessage(String message, String fNick, String fUID, String merUIDs){
        DatabaseReference msgRef = database.getRef().child("privateMessages").child(merUIDs);
        Map<String, Object> value = new HashMap<>();
        String key = msgRef.push().getKey();
        value.put("text",message);
        value.put("myNick",user.getDisplayName());
        value.put("user1ID",user.getUid());
        value.put("friendNick",fNick);
        value.put("user2ID",fUID);
        value.put("date", ServerValue.TIMESTAMP);
        value.put("msgKey",key);
        msgRef = database.getRef().child("privateMessages").child(merUIDs).child(key);
        msgRef.setValue(value);
        chatMessage.setText("");
    }
}
