package bakalarka.mendelu.cz.bakamsg;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.DataSetObserver;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.firebase.ui.database.FirebaseListAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import bakalarka.mendelu.cz.bakamsg.dataModels.Discussion;

public class Discussions extends AppCompatActivity {

    DatabaseReference database = FirebaseDatabase.getInstance().getReference();
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;
    final FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
    ListView discussionsList;
    private static final String TAG = "DiscussionsActivity";
    final Context context = this;
    ProgressDialog progressDialog;
    DatabaseReference numberOfDiscussions;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_discussions);
        discussionsList = (ListView)findViewById(R.id.list_discussions);
        numberOfDiscussions = database.child("discussions");

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                if (user != null) {
                    Log.w(TAG, " uživatel vešel jako " + user.getUid());
                } else {
                    Log.w(TAG, " uživatel se odhlásil ze služby");
                }
            }
        };
        /**
         * Naplnění dat z Firebase přes Firebase adapter do připraveného listu.
         */
        final FirebaseListAdapter<Discussion> adapter =
                new FirebaseListAdapter<Discussion>(this, Discussion.class, R.layout.list_discussions, database.child("discussions")) {
                    @Override
                    protected void populateView(View v, Discussion model, int position) {
                        ((TextView)v.findViewById(R.id.textViewTopic)).setText("Topic: "+model.getTopic());
                        ((TextView)v.findViewById(R.id.textViewOwnerOfTopic)).setText("Author: "+model.getOwner());
                    }
                };
        adapter.registerDataSetObserver(new DataSetObserver() {
            @Override
            public void onChanged() {
                super.onChanged();
                progressDialog.hide();
            }
        });
        discussionsList.setAdapter(adapter);
        discussionsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                openDiscussion(adapter.getItem(position));
            }
        });
        discussionsList.setLongClickable(true);
        discussionsList.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
            @Override
            public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int pos, long id) {
                final Discussion longClickDisc = adapter.getItem(pos);
                if (longClickDisc.getOwner().equals(user.getDisplayName())){
                    AlertDialog.Builder dialog = new AlertDialog.Builder(context);
                    dialog.setMessage("You are about to delete topic *"+longClickDisc.getTopic()+"*. Are you sure?");
                    dialog.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            DatabaseReference discRef = database.child("discussions/"+longClickDisc.getKey());
                            discRef.removeValue();
                            discRef = database.child("discussionMessages/"+longClickDisc.getKey());
                            discRef.removeValue();
                            dialog.dismiss();
                        }
                    });
                    dialog.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.dismiss();
                        }
                    });
                    AlertDialog aDialog = dialog.create();
                    aDialog.show();
                }
                return true;
            }
        });

        numberOfDiscussions.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.getChildrenCount()==0){
                    progressDialog.hide();
                }
                numberOfDiscussions.removeEventListener(this);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {}
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.discussion_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_addDiscussion:
                creatNewDicussion();
                break;
            case android.R.id.home:
                this.finish();
        }
        return true;
    }

    @Override
    public void onStart() {
        super.onStart();
        mAuth.addAuthStateListener(mAuthListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        if (mAuthListener != null) {
            mAuth.removeAuthStateListener(mAuthListener);
        }
    }
    /**
     * Otevření konkrétní diskuze.
     */
    private void openDiscussion(Discussion disc){
        Context context = this;
        Intent i = new Intent(context, SpecificDiscussion.class);
        i.putExtra("discussionKey",disc.getKey());
        i.putExtra("name",disc.getTopic());
        i.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(i);
    }
    /**
     * Založení nového vlákna v otevřených diskuzích.
     * Využívá se jednoduchého Dialogového rozhraní.
     */
    private void creatNewDicussion(){
        AlertDialog.Builder editMessage = new AlertDialog.Builder(context);
        final EditText edittext = new EditText(context);
        editMessage.setMessage("Create a new topic for discussing:");
        editMessage.setView(edittext);
        editMessage.setPositiveButton("Add", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                if (!edittext.getText().toString().isEmpty()){
                    DatabaseReference msgRef = database.child("discussions");
                    String key = msgRef.push().getKey();
                    Discussion disc = new Discussion(edittext.getText().toString(),user.getDisplayName(),key);
                    msgRef = database.child("discussions").child(key);
                    msgRef.setValue(disc);
                } else {
                    Toast.makeText(Discussions.this,R.string.dis_failed,Toast.LENGTH_LONG).show();
                }
            }
        });
        editMessage.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });
        editMessage.show();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }
}
