package bakalarka.mendelu.cz.bakamsg;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.database.DataSetObserver;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseListAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import bakalarka.mendelu.cz.bakamsg.dataModels.NewFriend;

public class FriendList extends AppCompatActivity {

    DatabaseReference database = FirebaseDatabase.getInstance().getReference();
    FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;
    private static final String TAG = "FriendListActivity";
    ListView friendList;
    DatabaseReference numberOfFriends;
    ProgressDialog progressDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_friend_list);
        friendList = (ListView)findViewById(R.id.list_friendlist);
        numberOfFriends = database.child("friends").child(user.getUid());

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                FirebaseUser user = firebaseAuth.getCurrentUser();
                if (user != null) {
                    Log.w(TAG, " uživatel je přihlášen jako " + user.getUid());
                } else {
                    Log.w(TAG, " uživatel se odhlásil ze služby");
                }
            }
        };
        /**
         * Naplnění dat z Firebase přes Firebase adapter do připraveného listu.
         */
        final FirebaseListAdapter<NewFriend> adapter =
            new FirebaseListAdapter<NewFriend>(this, NewFriend.class, R.layout.list_friend_requests, database.child("friends").child(user.getUid())) {
                @Override
                protected void populateView(View v, NewFriend model, int position) {
                    Log.w("Hodnoty ",user.getUid()+" "+model.getFriendKey());
                    ((TextView)v.findViewById(R.id.textViewFr)).setText(model.getNick());
                    ((TextView)v.findViewById(R.id.textViewFrMail)).setText(model.getEmail());
                }
            };

        adapter.registerDataSetObserver(new DataSetObserver() {
            @Override
            public void onChanged() {
                super.onChanged();
                progressDialog.hide();
            }
        });

        friendList.setAdapter(adapter);
        friendList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                openChatWithFriend(adapter.getItem(position));
            }
        });

        numberOfFriends.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.getChildrenCount()==0){
                    progressDialog.hide();
                }
                numberOfFriends.removeEventListener(this);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {}
        });
    }

    @Override
    public void onStart() {
        super.onStart();
        mAuth.addAuthStateListener(mAuthListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        if (mAuthListener != null) {
            mAuth.removeAuthStateListener(mAuthListener);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }
    /**
     * Otevření nové aktivity s chatovacím oknem mezi dvěma konkrétními uživateli.
     */
    private void openChatWithFriend(NewFriend fr){
        Log.w("Moje ID",user.getUid());
        Log.w("Jeho ID",fr.getFriendKey());
        Context context = this;
        Intent i = new Intent(context, ChatWindowWithFriend.class);
        i.putExtra("friendNick",fr.getNick());
        i.putExtra("friendUID",fr.getFriendKey());
        i.putExtra("chatWith",fr.getNick());
        i.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(i);
    }
}
