package bakalarka.mendelu.cz.bakamsg;

import android.app.ProgressDialog;
import android.database.DataSetObserver;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


import com.firebase.ui.database.FirebaseListAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import bakalarka.mendelu.cz.bakamsg.dataModels.NewFriend;
import bakalarka.mendelu.cz.bakamsg.dataModels.NewFriendRequest;
import bakalarka.mendelu.cz.bakamsg.dataModels.SimpleDialogFragment;
import bakalarka.mendelu.cz.bakamsg.dataModels.User;


public class FriendRequests extends AppCompatActivity {

    private static final String TAG = "FriendReuquests";
    static public String acceptedFriendRequest;
    static DatabaseReference database;
    static final FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
    DatabaseReference userRef = FirebaseDatabase.getInstance().getReference(); //2 zpusoby odkazovani do databaze
    DatabaseReference userReff;                                                //2 zpusoby odkazovani do databaze
    DatabaseReference numberOfReq;

    ListView listOfFriendsReq;
    Button buttonSendReq;
    EditText editTextNewRequest;
    ProgressDialog progressDialog;

    private boolean foundUser = false;
    long userCount;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_friend_requests);

        database = FirebaseDatabase.getInstance().getReference();
        userReff = database.getRef().child("users");
        numberOfReq = database.child("friendRequests").child(user.getUid());

        listOfFriendsReq = (ListView)findViewById(R.id.fr_list_req);
        buttonSendReq = (Button)findViewById(R.id.button_fr_req);
        editTextNewRequest = (EditText)findViewById(R.id.editTextNewRequest);

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        buttonSendReq.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendNewRequest();
            }
        });
        /**
         * Naplnění dat z Firebase přes Firebase adapter do připraveného listu.
         */
        final FirebaseListAdapter<NewFriendRequest> adapter =
            new FirebaseListAdapter<NewFriendRequest>(this, NewFriendRequest.class, R.layout.list_friend_requests, database.child("friendRequests").child(user.getUid())) {
                @Override
                protected void populateView(View v, NewFriendRequest model, int position) {
                    ((TextView)v.findViewById(R.id.textViewFr)).setText(model.getNick());
                    ((TextView)v.findViewById(R.id.textViewFrMail)).setText(model.getSourceEmail());
                }
            };

        adapter.registerDataSetObserver(new DataSetObserver() {
            @Override
            public void onChanged() {
                super.onChanged();
                progressDialog.hide();
            }
        });
        listOfFriendsReq.setAdapter(adapter);

        /**
         * Příprava na předání potřebných informací k přijetí nebo odmítnutí požadavku o přijetí do přátel.
         * Následně se zobrazí dialogový box.
         */
        listOfFriendsReq.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                Bundle bundle = new Bundle();
                NewFriendRequest nFr = adapter.getItem(position);
                bundle.putString("userName",nFr.getNick());
                bundle.putString("eMail",nFr.getSourceEmail());
                bundle.putString("UID",nFr.getUniqueID());
                bundle.putString("key",adapter.getRef(position).getKey());
                SimpleDialogFragment fragment = new SimpleDialogFragment();
                fragment.setArguments(bundle);
                fragment.show(getFragmentManager(),"Dialog");
            }
        });
        /**
         * Vložení pomocné informace o počtu uživatelů v databázi.
         */
        userReff.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                userCount = dataSnapshot.getChildrenCount();
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {}
        });

        numberOfReq.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.getChildrenCount()==0){
                    progressDialog.hide();
                    Toast.makeText(getApplicationContext(), "No requests have been found.", Toast.LENGTH_SHORT).show();
                }
                numberOfReq.removeEventListener(this);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {}
        });
    }

    /**
     * Odeslání nového požadavku o přidání do přátel s konkrétní e-mailovou adresou do databáze.
     * Než může transakce proběhnout, zkontroluje se, jestli daný uživatel existuje.
     * V případě že ano, vše proběhne hladce.
     * V případě, že je zadán e-mail aktuálně přihlášeného užřivatele, vypíše se o této skutečnosti informace.
     * A v případě, že uživatel se zadaným e-mailem v databázi není, je přihlášený uživatel informován.
     */
    private void sendNewRequest(){
        final String mail = editTextNewRequest.getText().toString();
        editTextNewRequest.setText("");

        if (TextUtils.isEmpty(mail)){
            editTextNewRequest.setError(getResources().getString(R.string.required));
        } else {
            foundUser = false;
            userReff.addChildEventListener(new ChildEventListener() {
                String targetUniqueID;
                long count = 0;
                @Override
                public void onChildAdded(DataSnapshot dataSnapshot, String s) {
                    User userr = dataSnapshot.getValue(User.class);
                    count++;
                    if (userr.getEmail().equals(mail) && mail!=user.getEmail()){
                        targetUniqueID = dataSnapshot.getKey();
                        NewFriendRequest newFriend = new NewFriendRequest(user.getUid(),user.getDisplayName(),user.getEmail(),mail);
                        userRef.child("friendRequests").child(targetUniqueID).push().setValue(newFriend);
                        editTextNewRequest.setError(null);
                        foundUser = true;
                        Toast.makeText(FriendRequests.this,"Friend request has been sent to the user.",Toast.LENGTH_LONG).show();
                        userReff.removeEventListener(this);
                    }
                    if (mail.equals(user.getEmail())){
                        Toast.makeText(FriendRequests.this,"You can't send request to your own e-mail.",Toast.LENGTH_LONG).show();
                    } else
                    if (count==userCount && !foundUser){
                        Toast.makeText(FriendRequests.this,"User with this e-mail does not exist.",Toast.LENGTH_LONG).show();
                    }
                }

                @Override
                public void onChildChanged(DataSnapshot dataSnapshot, String s) {}

                @Override
                public void onChildRemoved(DataSnapshot dataSnapshot) {}

                @Override
                public void onChildMoved(DataSnapshot dataSnapshot, String s) {}

                @Override
                public void onCancelled(DatabaseError databaseError) {}
            });
        }
    }
    /**
     * Přijetí požadavku o přidání do přátel.
     */
    public static void friendRequestAccepted(String UID, String key, String nick, String mail){
        DatabaseReference uRef = database.getRef().child("friends").child(user.getUid());
        /**
         * Vložení přítele pro přihlášeného uživatele.
         */
        NewFriend newFriend = new NewFriend(UID, nick, mail);
        uRef.push().setValue(newFriend);
        /**
         * Vložení přítele pro druhého uživatele.
         */
        uRef = database.getRef().child("friends").child(UID);
        newFriend = new NewFriend(user.getUid(), user.getDisplayName(), user.getEmail());
        uRef.push().setValue(newFriend);

        database.getRef().child("friendRequests").child(user.getUid()).child(key).removeValue();
    }

    /**
     * Odmítnutí požadavku o přijetí do přátel.
     */
    public static void friendRequestDeclined(String UID, String key){
        database.getRef().child("friendRequests").child(user.getUid()).child(key).removeValue();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }
}
