package bakalarka.mendelu.cz.bakamsg;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

public class LoginActivity extends AppCompatActivity {

    private static final String TAG = "LoginActivity";
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;

    private EditText e_mail;
    private EditText pass;

    private Button signInButton;
    private Button registerButton;

    private ProgressDialog progressDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        e_mail = (EditText) findViewById(R.id.mailEdit);
        pass = (EditText) findViewById(R.id.passEdit);

        signInButton = (Button) findViewById(R.id.sign_in);
        registerButton = (Button) findViewById(R.id.creatAcc);

        registerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToRegister(v);
            }
        });

        signInButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                signIn(e_mail.getText().toString(),pass.getText().toString());
            }
        });

        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
            FirebaseUser user = firebaseAuth.getCurrentUser();
            if (user != null) {
                /**
                 * Kontrola, která se volá v momentě, kdy se změní status uživatele, tedy, nově se přihlásí.
                 */
                Log.w(TAG, " uživatel je přihlášen jako " + user.getUid());
                goToUserPanel();
            } else {
                /**
                 * Totožné, v případě odhlášení.
                 */
                Log.w(TAG, " uživatel se odhlásil ze služby");
                }
            }
        };
    }

    @Override
    public void onStart() {
        super.onStart();
        mAuth.addAuthStateListener(mAuthListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        if (mAuthListener != null) {
            mAuth.removeAuthStateListener(mAuthListener);
        }
    }
    /**
     * Metoda volaná při znovuotevření aktivity.
     * Účel má pouze tehdy, když se uživatel vrací z aktivity Registrace po úspěšně založeném novém účtu.
     * V takovém případě oznámí jeho úspěšné založení uživateli.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode==2 && data!=null) {
            if (data.getBooleanExtra("Registered",false)){
                Toast.makeText(LoginActivity.this,R.string.acc_created,Toast.LENGTH_LONG).show();
            }
        }
    }
    /**
     * Metoda zpracovávající celý průběh přihlášení ke službě.
     */
    private void signIn(String email, String password){
        Log.w(TAG," přihlášení jako "+ email);
        /**
         * Kontrola vstupních parametrů.
         */
        if(!enteredTextFields()){
            return;
        }

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage("Logging in.");
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();
        /**
         * Proces přihlášení.
         */
        mAuth.signInWithEmailAndPassword(email,password)
            .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                @Override
                public void onComplete(@NonNull Task<AuthResult> task) {
                    if(!task.isSuccessful()){
                        Log.w(TAG," přihlášení se nezdařilo ",task.getException());
                        progressDialog.dismiss();
                        Toast.makeText(LoginActivity.this,R.string.login_failed,Toast.LENGTH_LONG).show();
                        return;
                    }
                    /*
                     * Pokud proces přihlášení skončí úspěšně, přechází se na aktivitu Userpanel.
                     */
                    if(task.isSuccessful()){
                        progressDialog.dismiss();
                        goToUserPanel();
                    }
                }
            });
    }
    /**
     * Otevření nové aktivity Registrace.
     */
    public void goToRegister(View v){
        Intent i = new Intent(LoginActivity.this,RegisterActivity.class);
        startActivityForResult(i,2);
    }
    /** Kontrolní funkce pro formulář, zkoumající, zdali nezůstalo některé pole nevyplněné.
     * V takovém případě ohlásí uživateli chybu.
     */
    private boolean enteredTextFields(){
        boolean isValid = true;
        String mail = e_mail.getText().toString();
        String ps1 = pass.getText().toString();

        if (TextUtils.isEmpty(mail)){
            e_mail.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            e_mail.setError(null);
        }

        if (TextUtils.isEmpty(ps1)){
            pass.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass.setError(null);
        }
        return isValid;
    }
    /**
     * Otevření aktivity Userpanel.
     */
    private void goToUserPanel(){
        Context context = this;
        Intent i = new Intent(context, UserpanelActivity.class);
        i.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(i);
    }
}
