package bakalarka.mendelu.cz.bakamsg;

import android.app.ProgressDialog;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.UserProfileChangeRequest;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import bakalarka.mendelu.cz.bakamsg.dataModels.User;


public class RegisterActivity extends AppCompatActivity {

    /**
     * Kontrolní proměnná s definicí aktuální aktivity pro výpis logů
     */
    private static final String TAG = "RegisterActivity";
    private FirebaseAuth mAuth;
    //private FirebaseAuth.AuthStateListener mAuthListener;
    private DatabaseReference datRef;

    private EditText e_mail;
    private EditText pass;
    private EditText pass2;
    private EditText alias;
    private ProgressDialog progressDialog;

    private Button register;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);

        datRef = FirebaseDatabase.getInstance().getReference();

        e_mail = (EditText) findViewById(R.id.emailEdit);
        pass = (EditText) findViewById(R.id.passEdit1);
        pass2 = (EditText) findViewById(R.id.passEdit2);
        alias = (EditText) findViewById(R.id.aliasEdit);
        getSupportActionBar().setTitle(getResources().getString(R.string.reg_form));

        progressDialog = new ProgressDialog(this);

        register = (Button) findViewById(R.id.buttonRegister);
        register.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                registerAccount(e_mail.getText().toString(), pass.getText().toString());
            }
        });

        // Zjišťování aktuálního stavu autentizace
        mAuth = FirebaseAuth.getInstance();
    }

    /** Metoda volaná v momentě vyplnění registračního formuláře.
     * Následně se provede kontrola zadaných informací. Pokud se nevyskytne žádný problém,
     * listener odešle do prostředí služby požadavek na vytvoření nového účtu.
     */
    public void registerAccount(String email, String password){
        Log.w(TAG, "vytváření ACC " + email);

        if(!enteredTextFields()){
            return;
        }

        progressDialog.setMessage("Creating new account.");
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        /**
         * Založení nového uživatele s přístupem ke službě.
         */
        mAuth.createUserWithEmailAndPassword(email,password)
            .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                @Override
                public void onComplete(@NonNull Task<AuthResult> task) {
                    Log.w(TAG, "Vytvoreni uzivatele "+ task.isSuccessful());
                    /**
                     * Vložení hodnoty uživatele do databáze kvůli pomocným funkcím v jiných aktivitách
                     */
                    if(task.isSuccessful()){
                        createNew(task.getResult().getUser());
                    }
                    /**
                     * Pokud během zpracování dojde k přerušení, vypíše se varovná hláška
                     */
                    if (!task.isSuccessful()){
                        progressDialog.hide();
                        Toast.makeText(RegisterActivity.this,R.string.auth_failed,Toast.LENGTH_LONG).show();
                        return;
                    }
                }
            });
    }

    /** Metoda, která zahájí tvořící proces uživatele jako pomocné proměnné v databázi.
     * @param u vytažení současného uživatele (toho, který prošel registrací)
     */
    private void createNew(FirebaseUser u){
        final FirebaseUser user = u;
        //final String name = creatUserName(user.getEmail());

        UserProfileChangeRequest profileUpdates = new UserProfileChangeRequest.Builder()
                .setDisplayName(alias.getText().toString())
                .build();
        user.updateProfile(profileUpdates)
                .addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        creatUserInDatabase(user.getUid(),alias.getText().toString(),user.getEmail());
                    }
                });
    }

    // Defaultní tvorba uživatelského jména podle vložené e-mailové adresy
    /*private String creatUserName(String s){
        if (s.contains("@")){
            return s.split("@")[0];
        } else {
            return s;
        }
    }*/

    @Override
    protected void onDestroy(){
        super.onDestroy();
        progressDialog.dismiss();
    }

    /**
     * Založení nového uživatele a odeslání jeho informací pro pozdější účely.
     * Toto nevytváří účet, ale pouze předává jeho základní informace do stromové databáze.
     */
    private void creatUserInDatabase(String ID, String name, String email){
        User user = new User(name, email);
        datRef.child("users").child(ID).setValue(user);

        /**
         * Aktivita se otevírá jako potomek, jakmile dojde k úspěšnému odeslání požadavku na vytvoření nového účtu,
         * aktivita se zavře a předá se hodnota RESULTU do aktivity předchozí, kde se následně zobrazí Toast oznamující
         * uživateli úspěšně provedenou transakci.
         */
        FirebaseAuth.getInstance().signOut();
        Intent result = new Intent();
        result.putExtra("Registered",true);
        setResult(2,result);
        progressDialog.dismiss();
        finish();
    }

    /** Kontrola zadaných informací, zda vyhovují předepsaným pravidlům.
     *  Služba požaduje při ručním vytváření účtu délku hesla alespoň 6 znaků.
     */
    private boolean enteredTextFields(){
        boolean isValid = true;

        String mail = e_mail.getText().toString();
        String ps1 = pass.getText().toString();
        String ps2 = pass2.getText().toString();
        String aliass = alias.getText().toString();

        if (TextUtils.isEmpty(mail)){
            e_mail.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            e_mail.setError(null);
        }

        if (TextUtils.isEmpty(aliass)){
            alias.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            alias.setError(null);
        }

        if (TextUtils.isEmpty(ps1)){
            pass.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass.setError(null);
        }

        if (TextUtils.isEmpty(ps2)){
            pass2.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass2.setError(null);
        }

        if (aliass.length()<=3 || aliass.length()>=21){
            alias.setError(getResources().getString(R.string.nick_length));
            isValid = false;
        } else {
            alias.setError(null);
        }

        if (ps1.length()<6){
            pass.setError(getResources().getString(R.string.pass_length));
            isValid = false;
        } else {
            pass.setError(null);
        }

        if (!new String(ps1).equals(ps2)){
            pass2.setError(getResources().getString(R.string.pass_error));
            isValid = false;
        } else {
            pass2.setError(null);
        }
        return isValid;
    }
}

