package bakalarka.mendelu.cz.bakamsg;

import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;

import com.firebase.ui.database.FirebaseListAdapter;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.ServerValue;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import bakalarka.mendelu.cz.bakamsg.dataModels.Message;

public class SpecificDiscussion extends AppCompatActivity {

    DatabaseReference database = FirebaseDatabase.getInstance().getReference();
    final FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
    String discussionKey;
    Button sendButton;
    EditText discussionMessage;
    ListView listOfMessages;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_specific_discussion);
        Intent i = getIntent();
        discussionKey = i.getStringExtra("discussionKey");
        getSupportActionBar().setTitle(i.getStringExtra("name"));

        sendButton = (Button) findViewById(R.id.buttonDiscussion);
        discussionMessage = (EditText) findViewById(R.id.editTextDiscussion);
        listOfMessages = (ListView) findViewById(R.id.listOfDiscussionMessages);
        /**
         * Příprava na převod časového razítka ze serveru na lokální čas.
         */
        final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        final String timezoneID = TimeZone.getDefault().getID();
        dateFormat.setTimeZone(TimeZone.getTimeZone(timezoneID));

        sendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!discussionMessage.getText().toString().isEmpty()){
                    sendMessage(discussionMessage.getText().toString());
                }
            }
        });
        /**
         * Naplnění dat z Firebase přes Firebase adapter do připraveného listu.
         */
        final FirebaseListAdapter<Message> adapter =
            new FirebaseListAdapter<Message>(this, Message.class, R.layout.list_chat_messages, database.child("discussionMessages").child(discussionKey).orderByChild("date")) {
                @Override
                protected void populateView(View v, Message model, int position) {
                    ((TextView)v.findViewById(R.id.textViewListMessage)).setText(model.getText());
                    ((TextView)v.findViewById(R.id.textViewListNick)).setText("From: "+model.getMyNick());
                    ((TextView)v.findViewById(R.id.textViewListDate)).setText("["+dateFormat.format(new Date (model.getDate()))+"]");
                }
            };

        listOfMessages.setAdapter(adapter);
    }
    /**
     * Odeslání zprávy do Firebase.
     */
    public void sendMessage(String message){
        DatabaseReference msgRef;
        Map<String, Object> value = new HashMap<>();
        value.put("text",message);
        value.put("myNick",user.getDisplayName());
        value.put("user1ID",user.getUid());
        value.put("date", ServerValue.TIMESTAMP);
        msgRef = database.getRef().child("discussionMessages").child(discussionKey);
        msgRef.push().setValue(value);
        discussionMessage.setText("");
    }
}