package bakalarka.mendelu.cz.bakamsg;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.util.Log;
import android.view.View;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.TextView;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;

public class UserpanelActivity extends AppCompatActivity {

    private static final String TAG = "UserPanelActivity";
    private FirebaseAuth mAuth;
    private FirebaseAuth.AuthStateListener mAuthListener;

    private TextView textViewShowUser;
    private Button buttonFriend;
    private Button buttonDiscussions;
    private Button buttonFriendRequests;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_usrpanel);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        textViewShowUser = (TextView) findViewById(R.id.textViewLoggedUser);
        buttonDiscussions = (Button) findViewById(R.id.buttonDiscussion);
        buttonFriend = (Button) findViewById(R.id.buttonFriends);
        buttonFriendRequests = (Button) findViewById(R.id.buttonFriendReq);
        FirebaseUser user = FirebaseAuth.getInstance().getCurrentUser();
        textViewShowUser.setText(getString(R.string.logged)+" "+user.getDisplayName());

        buttonFriend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToAnotherActivity("FriendsActivity");
            }
        });
        buttonDiscussions.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToAnotherActivity("DiscussionsActivity");
            }
        });
        buttonFriendRequests.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToAnotherActivity("FriendRequestsActivity");
            }
        });

        mAuth = FirebaseAuth.getInstance();
        mAuthListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                FirebaseUser user = firebaseAuth.getCurrentUser();
                if (user != null) {
                    /**
                     * Kontrola, která se volá v momentě, kdy se změní status uživatele, tedy, nově se přihlásí.
                     */
                    Log.w(TAG, " uživatel je přihlášen jako " + user.getUid());
                } else {
                    /**
                     * Totožné, v případě odhlášení.
                     */
                    Log.w(TAG, " uživatel se odhlásil ze služby");
                }
            }
        };
    }

    @Override
    public void onStart() {
        super.onStart();
        mAuth.addAuthStateListener(mAuthListener);
    }

    @Override
    public void onStop() {
        super.onStop();
        if (mAuthListener != null) {
            mAuth.removeAuthStateListener(mAuthListener);
        }
    }
    /**
     * Rozcestník.
     */
    private void goToAnotherActivity(String activity){
        Context context = this;
        Intent i = null;
        switch (activity) {
            case "FriendsActivity":
                i = new Intent(context, FriendList.class);
                break;
            case "FriendRequestsActivity":
                i = new Intent(context, FriendRequests.class);
                break;
            case "DiscussionsActivity":
                i = new Intent(context, Discussions.class);
                break;
        }
        i.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(i);
    }
    /**
     * Zakázání funkce jít zpět v zařízení – důvodem je nemožnost vrátit se zpět do Login activity jinak než přes korektní odhlášení se ze služby.
     */
    @Override
    public void onBackPressed() {
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.userpanel, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_logout) {
            signOut();
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * Odhlášení ze služby.
     */
    private void signOut(){
        FirebaseAuth.getInstance().signOut();
        finish();
    }
}
