package bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.kinvey.android.AsyncAppData;
import com.kinvey.android.Client;
import com.kinvey.android.callback.KinveyListCallback;
import com.kinvey.java.Query;
import com.kinvey.java.core.KinveyClientCallback;
import com.kinvey.java.query.AbstractQuery;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;
import java.util.Timer;
import java.util.TimerTask;

import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.Adapters.MessageAdapter;
import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.dataModels.Message;


public class ChatWindowWithFriend extends AppCompatActivity {

    private final String TAG = "ChatWindowWithFriend";
    Client mKinveyClient;
    Button sendButton;
    EditText chatMessage;
    ListView listOfMessages;
    String friendNick;
    String myNick;
    final Context context = this;
    ProgressDialog progressDialog;
    MessageAdapter adapter;
    ArrayList<Message> msgArray = new ArrayList<>();
    final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    final String timezoneID = TimeZone.getDefault().getID();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chat_window_with_friend);

        sendButton = (Button) findViewById(R.id.buttonDiscussion);
        chatMessage = (EditText) findViewById(R.id.editTextDiscussion);
        listOfMessages = (ListView) findViewById(R.id.listOfChatMessages);

        mKinveyClient = new Client.Builder(this.getApplicationContext()).build();

        Intent i = getIntent();
        friendNick = i.getStringExtra("friendNick");
        myNick = i.getStringExtra("myNick");
        getSupportActionBar().setTitle("Chat with "+i.getStringExtra("chatWith"));

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        /**
         * Příprava na převod časového razítka ze serveru na lokální čas.
         */
        dateFormat.setTimeZone(TimeZone.getTimeZone(timezoneID));

        setAdapter();

        sendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!chatMessage.getText().toString().isEmpty()){
                    sendMessage(chatMessage.getText().toString(),friendNick,myNick);
                }
            }
        });
        /**
         * Editace existujících zpráv, jichž je přihlášený uživatel vlastníkem.
         */
        listOfMessages.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, final int position, long id) {
                final Message msg = adapter.getItem(position);
                if (msg.getMyNick().equals(myNick)){
                    AlertDialog.Builder editMessage = new AlertDialog.Builder(context);
                    final EditText edittext = new EditText(context);
                    editMessage.setMessage("Original: "+msg.getMessage());
                    editMessage.setView(edittext);
                    editMessage.setPositiveButton("Edit", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                            msg.setMessage(edittext.getText().toString());
                            if (!msg.getMessage().equals("")){
                                AsyncAppData<Message> upMsg = mKinveyClient.appData("friendMessages", Message.class);
                                upMsg.save(msg, new KinveyClientCallback<Message>() {
                                    @Override
                                    public void onSuccess(Message r) {
                                        Toast.makeText(getApplicationContext(), "Message has been edited.", Toast.LENGTH_SHORT).show();
                                        Log.w(TAG, "Data ulozena.");
                                        adapter.notifyDataSetChanged();
                                    }

                                    @Override
                                    public void onFailure(Throwable e) {
                                        Log.e(TAG, "Nepodarilo se odeslat zpravu.", e);
                                    }
                                });
                            } else {
                                Toast.makeText(getApplicationContext(), "Message has not been edited.", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
                    editMessage.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                        }
                    });
                    editMessage.show();
                }
            }
        });
    }
    /*
     * Naplnění aktuálních dat ze serveru do listview.
     */
    public void callForData(){
        Query q;
        Query query1 = new Query().equals("friendNick",friendNick);
        Query query2 = new Query().equals("myNick",myNick);
        Query query3 = new Query().equals("friendNick",myNick);
        Query query4 = new Query().equals("myNick",friendNick);
        q = (query1.and(query2)).or(query3.and(query4));
        q.addSort("date", AbstractQuery.SortOrder.ASC);
        AsyncAppData<Message> data = mKinveyClient.appData("friendMessages", Message.class);
        data.get(q, new KinveyListCallback<Message>() {
            @Override
            public void onSuccess(Message[] msgs) {
                msgArray.clear();
                adapter.addAll(msgs);
                adapter.notifyDataSetChanged();
                progressDialog.hide();
            }

            @Override
            public void onFailure(Throwable t) {
                progressDialog.hide();
            }
        });
    }
    /**
     * Naplnění dat z Kinveyho do připraveného listu.
     */
    public void setAdapter(){
        adapter = new MessageAdapter(this, msgArray);
        listOfMessages.setAdapter(adapter);
        callForData();
        startNewThread();
    }
    /*
     * Vytvoření nového vlákna pro opakování volání na server a update zpráv v určitém intervalu.
     */
    public void startNewThread(){
        new Thread(new Runnable() {
            public void run() {
                new Timer().scheduleAtFixedRate(new TimerTask() {
                    @Override
                    public void run() {
                        callForData();
                    }
                },5000,3000);
            }
        }).start();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }
    /**
     * Odeslání zprávy do Kinveyho.
     */
    public void sendMessage(String message, String fNick, String mNick){
        String dateAndTime = dateFormat.format(new Date());
        progressDialog.setMessage("Sending, please wait.");
        Message msg = new Message(message,fNick,mNick,dateAndTime);
        AsyncAppData<Message> frs = mKinveyClient.appData("friendMessages",Message.class);
        frs.save(msg, new KinveyClientCallback<Message>() {
            @Override
            public void onSuccess(Message msgs) {
                Toast.makeText(getApplicationContext(), "Message has been sent.", Toast.LENGTH_SHORT).show();
                Log.w(TAG, "Data ulozena.");
                chatMessage.setText("");
                adapter.add(msgs);
                progressDialog.setMessage(getResources().getString(R.string.loading));
                progressDialog.hide();
            }
            @Override
            public void onFailure(Throwable e) {
                Log.e(TAG, "Nepodarilo se odeslat zpravu.", e);
                progressDialog.hide();
            }
        });
    }
}
