package bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.kinvey.android.AsyncAppData;
import com.kinvey.android.Client;
import com.kinvey.android.callback.KinveyDeleteCallback;
import com.kinvey.android.callback.KinveyListCallback;
import com.kinvey.java.Query;
import com.kinvey.java.core.KinveyClientCallback;
import com.kinvey.java.model.KinveyDeleteResponse;

import java.util.ArrayList;

import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.Adapters.DiscussionAdapter;
import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.dataModels.Discussion;
import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.dataModels.DiscussionMessage;


public class Discussions extends AppCompatActivity {
    Client mKinveyClient;
    private final String TAG = "DiscussionsActivity";
    ListView discussionsList;
    DiscussionAdapter adapter;
    ProgressDialog progressDialog;
    final ArrayList<Discussion> discussions = new ArrayList<>();
    final Context context = this;

    String username;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_discussions);
        discussionsList = (ListView)findViewById(R.id.list_discussions);
        mKinveyClient = new Client.Builder(this.getApplicationContext()).build();

        username = UserpanelActivity.username;
        Log.w(TAG,username);

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        /**
         * Naplnění dat z Kinveyho do připraveného listu.
         */
        Query q = mKinveyClient.query();
        AsyncAppData<Discussion> data = mKinveyClient.appData("discussions", Discussion.class);
        data.get(q, new KinveyListCallback<Discussion>() {
            @Override
            public void onSuccess(Discussion[] discs) {
                Log.w(TAG, "Pocet najitych " + discs.length);
                for(int i=0; i<=discs.length-1; i++){
                    discussions.add(discs[i]);
                }
                setAdapter();
            }
            @Override
            public void onFailure(Throwable t) {
                progressDialog.hide();
            }
        });

        discussionsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                openDiscussion(adapter.getItem(position));
            }
        });
        /*
         * Nastavení funkce longclick pro list, aby bylo možné jednotlivé diskuse smazat.
         * Společně s diskusí se mažou i patřičné zprávy.
         */
        discussionsList.setLongClickable(true);
        discussionsList.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
            @Override
            public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int pos, long id) {
                final Discussion longClickDisc = adapter.getItem(pos);
                if (longClickDisc.getOwner().equals(username)){
                    final Query query1 = new Query().equals("discId",longClickDisc.getId());
                    final Query query2 = new Query().equals("_id",longClickDisc.getId());
                    AlertDialog.Builder dialog = new AlertDialog.Builder(context);
                    dialog.setMessage("You are about to delete topic *"+longClickDisc.getTopic()+"*. Are you sure?");
                    dialog.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            AsyncAppData<DiscussionMessage> delDiscussionMessages = mKinveyClient.appData("discussionMessages", DiscussionMessage.class);
                            delDiscussionMessages.delete(query1, new KinveyDeleteCallback() {
                                @Override
                                public void onSuccess(KinveyDeleteResponse response) {
                                    Log.w(TAG, "Zprávy byly smazány.");

                                    AsyncAppData<Discussion> delDiscussion = mKinveyClient.appData("discussions", Discussion.class);
                                    delDiscussion.delete(query2, new KinveyDeleteCallback() {
                                        @Override
                                        public void onSuccess(KinveyDeleteResponse response) {
                                            Log.w(TAG, "Diskuse byla smazána.");
                                            adapter.remove(longClickDisc);     //mazani z listu narychlo bez dotazu na server
                                            adapter.notifyDataSetChanged();  //mazani z listu narychlo bez dotazu na server
                                        }
                                        public void onFailure(Throwable e) {
                                            Log.e(TAG, "Nepodařilo se diskusi smazat. ", e);
                                        }
                                    });
                                }

                                public void onFailure(Throwable e) {
                                    Log.e(TAG, "Nepodařilo se zprávy smazat. ", e);
                                }
                            });
                            dialog.dismiss();
                            // Long tap on discussion you created for deletion.
                        }
                    });
                    dialog.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.dismiss();
                        }
                    });
                    AlertDialog aDialog = dialog.create();
                    aDialog.show();
                }
                return true;
            }
        });
    }

    public void setAdapter(){
        progressDialog.hide();
        adapter = new DiscussionAdapter(this, discussions);
        discussionsList.setAdapter(adapter);
    }

    @Override
    public void onStart() {
        super.onStart();
    }

    @Override
    public void onStop() {
        super.onStop();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.discussion_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_addDiscussion:
                creatNewDicussion();
                break;
            case android.R.id.home:
                this.finish();
        }
        return true;
    }
    /**
     * Otevření konkrétní diskuze.
     */
    private void openDiscussion(Discussion disc){
        Context context = this;
        Intent i = new Intent(context, SpecificDiscussion.class);
        i.putExtra("discussionId",disc.getId());
        i.putExtra("topicName",disc.getTopic());
        i.putExtra("username",username);
        i.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
        startActivity(i);
    }
    /**
     * Založení nového vlákna v otevřených diskuzích.
     * Využívá se jednoduchého Dialogového rozhraní.
     */
    private void creatNewDicussion(){
        AlertDialog.Builder editMessage = new AlertDialog.Builder(context);
        final EditText edittext = new EditText(context);
        editMessage.setMessage("Create a new topic for discussing:");
        editMessage.setView(edittext);
        editMessage.setPositiveButton("Add", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                if (!edittext.getText().toString().isEmpty()){

                    Discussion dsc = new Discussion(edittext.getText().toString(),username);
                    AsyncAppData<Discussion> disc = mKinveyClient.appData("discussions",Discussion.class);
                    disc.save(dsc, new KinveyClientCallback<Discussion>() {
                        @Override
                        public void onSuccess(Discussion newDiscussion) {
                            adapter.add(newDiscussion);//ruční vložení do adaptéru
                            Toast.makeText(getApplicationContext(), "Topic has been created.", Toast.LENGTH_SHORT).show();
                            Log.e("TAG", "Data ulozena.");
                            progressDialog.hide();
                        }
                        @Override
                        public void onFailure(Throwable e) {
                            Log.e("TAG", "Nepodarilo se ulozit friend request data", e);
                            progressDialog.hide();
                        }
                    });
                } else {
                    Toast.makeText(Discussions.this,R.string.dis_failed,Toast.LENGTH_LONG).show();
                }
            }
        });
        editMessage.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
            }
        });
        editMessage.show();
    }
}
