package bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey;

import android.app.ProgressDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.kinvey.android.Client;
import com.kinvey.android.callback.KinveyPingCallback;
import com.kinvey.android.callback.KinveyUserCallback;
import com.kinvey.java.User;

public class LoginActivity extends AppCompatActivity {

    private static final String TAG = "LoginActivity";
    private EditText alias;
    private EditText pass;

    private Button signInButton;
    private Button registerButton;

    private ProgressDialog progressDialog;
    Client mKinveyClient;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        mKinveyClient = new Client.Builder(this.getApplicationContext()).build();
        alias = (EditText) findViewById(R.id.mailEdit);
        pass = (EditText) findViewById(R.id.passEdit);

        signInButton = (Button) findViewById(R.id.sign_in);
        registerButton = (Button) findViewById(R.id.creatAcc);

        registerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToRegister(v);
            }
        });

        signInButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                signIn(alias.getText().toString(),pass.getText().toString());
            }
        });

        if (mKinveyClient.user().isUserLoggedIn()){
            goToUserPanel();
        } else{
            /**
             * Kontrola spojení s backendem.
             */
            mKinveyClient.ping(new KinveyPingCallback() {
                public void onFailure(Throwable t) {
                    Log.w(TAG, " Kinvey se nespojil.", t);
                }
                public void onSuccess(Boolean b) {
                    Log.w(TAG, " Kinvey se úspěšně spojil.");
                }
            });
        }
    }
    /**
     * Metoda volaná při znovuotevření aktivity.
     * Účel má pouze tehdy, když se uživatel vrací z aktivity Registrace po úspěšně založeném novém účtu.
     * V takovém případě oznámí jeho úspěšné založení uživateli.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode==2 && data!=null) {
            if (data.getBooleanExtra("Registered",false)){
                Toast.makeText(LoginActivity.this,R.string.acc_created,Toast.LENGTH_LONG).show();
            }
        }
        if(requestCode==3 && data!=null) {
            if (data.getBooleanExtra("Logged-out",false)){
                mKinveyClient.user().logout().execute();
            }
        }
    }
    /**
     * Metoda zpracovávající celý průběh přihlášení ke službě.
     */
    private void signIn(String alias, String password){
        mKinveyClient.user().logout().execute();
        Log.w(TAG," přihlášení jako "+ alias);
        /**
         * Kontrola vstupních parametrů.
         */
        if(!enteredTextFields()){
            return;
        }

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage("Logging in.");
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();
        /**
         * Proces přihlášení.
         */
        mKinveyClient.user().login(alias, password, new KinveyUserCallback() {
            @Override
            public void onSuccess(User user) {
                CharSequence text = "Welcome back, " + user.getUsername() + ".";
                Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT).show();
                progressDialog.dismiss();
                goToUserPanel();
            }

            @Override
            public void onFailure(Throwable t) {
                CharSequence text = "Wrong username or password.";
                Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT).show();
                progressDialog.dismiss();
            }
        });

    }
    /**
     * Otevření nové aktivity Registrace.
     */
    public void goToRegister(View v){
        Intent i = new Intent(LoginActivity.this,RegisterActivity.class);
        startActivityForResult(i,2);
    }
    /** Kontrolní funkce pro formulář, zkoumající, zdali nezůstalo některé pole nevyplněné.
     * V takovém případě ohlásí uživateli chybu.
     */
    private boolean enteredTextFields(){
        boolean isValid = true;
        String mail = alias.getText().toString();
        String ps1 = pass.getText().toString();

        if (TextUtils.isEmpty(mail)){
            alias.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            alias.setError(null);
        }

        if (TextUtils.isEmpty(ps1)){
            pass.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass.setError(null);
        }
        return isValid;
    }
    /**
     * Otevření aktivity UserPanel.
     */
    private void goToUserPanel(){
        Intent i = new Intent(LoginActivity.this,UserpanelActivity.class);
        startActivityForResult(i,3);
    }
}
