package bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey;

import android.support.v7.app.AppCompatActivity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.kinvey.android.Client;
import com.kinvey.android.callback.KinveyUserCallback;
import com.kinvey.java.User;

public class RegisterActivity extends AppCompatActivity {

    /**
     * Kontrolní proměnná s definicí aktuální aktivity pro výpis logů
     */
    private static final String TAG = "RegisterActivity";
    Client mKinveyClient;

    private EditText e_mail;
    private EditText pass;
    private EditText pass2;
    private EditText alias;
    private ProgressDialog progressDialog;

    private Button register;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);
        mKinveyClient = new Client.Builder(this.getApplicationContext()).build();

        e_mail = (EditText) findViewById(R.id.emailEdit);
        pass = (EditText) findViewById(R.id.passEdit1);
        pass2 = (EditText) findViewById(R.id.passEdit2);
        alias = (EditText) findViewById(R.id.aliasEdit);
        getSupportActionBar().setTitle(getResources().getString(R.string.reg_form));

        progressDialog = new ProgressDialog(this);

        register = (Button) findViewById(R.id.buttonRegister);
        register.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                registerAccount(alias.getText().toString(), pass.getText().toString());
            }
        });

    }

    /**
     * Metoda volaná v momentě vyplnění registračního formuláře.
     * Následně se provede kontrola zadaných informací. Pokud se nevyskytne žádný problém,
     * listener odešle do prostředí služby požadavek na vytvoření nového účtu.
     */
    public void registerAccount(String alias, String password){
        Log.w(TAG, "vytváření ACC " + alias);

        if(!enteredTextFields()){
            return;
        }

        progressDialog.setMessage("Creating new account.");
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        /**
         * Založení nového uživatele s přístupem ke službě.
         */
        mKinveyClient.user().create(alias, password, new KinveyUserCallback() {
            @Override
            public void onSuccess(User user) {
                mKinveyClient.user().put("email",e_mail.getText().toString());
                mKinveyClient.user().update(new KinveyUserCallback() {
                    @Override
                    public void onFailure(Throwable e) { }
                    @Override
                    public void onSuccess(User u) {
                        Log.w(TAG, " username: "+u.getUsername());
                        Intent result = new Intent();
                        result.putExtra("Registered",true);
                        setResult(2,result);
                        progressDialog.dismiss();
                        mKinveyClient.user().logout().execute();
                        finish();
                    }
                });
            }
            @Override
            public void onFailure(Throwable throwable) {
                CharSequence text = "Could not sign up.";
                Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT).show();
                progressDialog.dismiss();
            }
        });
    }

    @Override
    protected void onDestroy(){
        super.onDestroy();
        progressDialog.dismiss();
    }
    /** Kontrola zadaných informací, zda vyhovují předepsaným pravidlům.
     *  Služba požaduje při ručním vytváření účtu délku hesla alespoň 6 znaků.
     */
    private boolean enteredTextFields(){
        boolean isValid = true;

        String mail = e_mail.getText().toString();
        String ps1 = pass.getText().toString();
        String ps2 = pass2.getText().toString();
        String aliass = alias.getText().toString();

        if (TextUtils.isEmpty(mail)){
            e_mail.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            e_mail.setError(null);
        }

        if (TextUtils.isEmpty(aliass)){
            alias.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            alias.setError(null);
        }

        if (TextUtils.isEmpty(ps1)){
            pass.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass.setError(null);
        }

        if (TextUtils.isEmpty(ps2)){
            pass2.setError(getResources().getString(R.string.required));
            isValid = false;
        } else {
            pass2.setError(null);
        }

        if (aliass.length()<=3 || aliass.length()>=21){
            alias.setError(getResources().getString(R.string.nick_length));
            isValid = false;
        } else {
            alias.setError(null);
        }

        if (ps1.length()<6){
            pass.setError(getResources().getString(R.string.pass_length));
            isValid = false;
        } else {
            pass.setError(null);
        }

        if (!new String(ps1).equals(ps2)){
            pass2.setError(getResources().getString(R.string.pass_error));
            isValid = false;
        } else {
            pass2.setError(null);
        }
        return isValid;
    }

}

