package bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.kinvey.android.AsyncAppData;
import com.kinvey.android.Client;
import com.kinvey.android.callback.KinveyListCallback;
import com.kinvey.java.Query;
import com.kinvey.java.core.KinveyClientCallback;
import com.kinvey.java.query.AbstractQuery;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;
import java.util.Timer;
import java.util.TimerTask;

import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.Adapters.DiscussionMessageAdapter;
import bakamsgkinvey.cz.mendelu.bakalarka.bakamsgkinvey.dataModels.DiscussionMessage;

public class SpecificDiscussion extends AppCompatActivity {
    private final String TAG = "SpecificDiscussion";
    Client mKinveyClient;
    DiscussionMessageAdapter adapter;
    ArrayList<DiscussionMessage> msgArray = new ArrayList<>();

    String discussionKey;
    String username;
    Button sendButton;
    EditText discussionMessage;
    ListView listOfMessages;
    ProgressDialog progressDialog;
    final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    final String timezoneID = TimeZone.getDefault().getID();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_specific_discussion);

        mKinveyClient = new Client.Builder(this.getApplicationContext()).build();
        Intent i = getIntent();
        discussionKey = i.getStringExtra("discussionId");
        username = i.getStringExtra("username");
        getSupportActionBar().setTitle(i.getStringExtra("topicName"));

        sendButton = (Button) findViewById(R.id.buttonDiscussion);
        discussionMessage = (EditText) findViewById(R.id.editTextDiscussion);
        listOfMessages = (ListView) findViewById(R.id.listOfDiscussionMessages);

        progressDialog = new ProgressDialog(this);
        progressDialog.setMessage(getResources().getString(R.string.loading));
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();

        /**
         * Příprava na převod časového razítka ze serveru na lokální čas.
         */
        dateFormat.setTimeZone(TimeZone.getTimeZone(timezoneID));

        setAdapter();
        callForData();

        sendButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!discussionMessage.getText().toString().isEmpty()){
                    sendMessage(discussionMessage.getText().toString());
                }
            }
        });

        listOfMessages.setAdapter(adapter);
    }
    /*
     * Naplnění aktuálních dat ze serveru do listview.
     */
    public void callForData(){
        Query q = mKinveyClient.query();
        q.equals("discId",discussionKey);
        q.addSort("date", AbstractQuery.SortOrder.ASC);
        AsyncAppData<DiscussionMessage> data = mKinveyClient.appData("discussionMessages", DiscussionMessage.class);
        data.get(q, new KinveyListCallback<DiscussionMessage>() {
            @Override
            public void onSuccess(DiscussionMessage[] msgs) {
                msgArray.clear();
                adapter.addAll(msgs);
                adapter.notifyDataSetChanged();
                progressDialog.hide();
            }

            @Override
            public void onFailure(Throwable t) {
                progressDialog.hide();
            }
        });
    }
    /**
     * Naplnění dat z Kinveyho do připraveného listu.
     */
    public void setAdapter(){
        adapter = new DiscussionMessageAdapter(this, msgArray);
        listOfMessages.setAdapter(adapter);
        startNewThread();
    }
    /*
     * Vytvoření nového vlákna pro opakování volání na server a update zpráv v určitém intervalu.
     */
    public void startNewThread(){
        new Thread(new Runnable() {
            public void run() {
                new Timer().scheduleAtFixedRate(new TimerTask() {
                    @Override
                    public void run() {
                        callForData();
                    }
                },5000,3000);
            }
        }).start();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        progressDialog.dismiss();
    }
    /**
     * Odeslání zprávy do Kinveyho.
     */
    public void sendMessage(String message){
        String dateAndTime = dateFormat.format(new Date());
        progressDialog.setMessage("Sending, please wait.");
        DiscussionMessage msg = new DiscussionMessage(message,username,dateAndTime,discussionKey);
        AsyncAppData<DiscussionMessage> frs = mKinveyClient.appData("discussionMessages",DiscussionMessage.class);
        frs.save(msg, new KinveyClientCallback<DiscussionMessage>() {
            @Override
            public void onSuccess(DiscussionMessage msgs) {
                Toast.makeText(getApplicationContext(), "Message has been sent.", Toast.LENGTH_SHORT).show();
                Log.w(TAG, "Data ulozena.");
                discussionMessage.setText("");
                adapter.add(msgs);
                progressDialog.setMessage(getResources().getString(R.string.loading));
                progressDialog.hide();
            }
            @Override
            public void onFailure(Throwable e) {
                Log.e(TAG, "Nepodarilo se odeslat zpravu.", e);
                progressDialog.hide();
            }
        });
        discussionMessage.setText("");
    }
}